import fs from 'fs/promises';
import path from 'path';
import lunr from 'lunr';
import { getRelativeUrl } from '../../utilities/file.js';
const map = [];
function stripWhitespace(string) {
    return string.replace(/\s+/g, ' ');
}
export default function (options) {
    const opts = {
        ignore: () => false,
        getTitle: doc => doc.querySelector('title')?.textContent?.trim() ?? '',
        getDescription: doc => doc.querySelector('meta[name="description"]')?.getAttribute('content') ?? '',
        getHeadings: doc => [...doc.querySelectorAll('h2, h3, h4')].map(heading => heading.textContent?.trim() ?? ''),
        getContent: doc => doc.querySelector('body')?.textContent?.trim() || '',
        searchDirName: 'search',
        ...options
    };
    let pagesToIndex = [];
    return {
        transform: (doc, page, config) => {
            const searchDir = path.join(config.outputDir, config.assetFolderName, opts.searchDirName);
            const clientScriptFilename = path.join(searchDir, 'search-client.js');
            const clientStylesFilename = path.join(searchDir, 'search-client.css');
            const sidebarAddons = doc.querySelector('#docs-sidebar-addons');
            const clientScript = doc.createElement('script');
            clientScript.src = getRelativeUrl(config, clientScriptFilename);
            clientScript.defer = true;
            doc.body.append(clientScript);
            const clientStyles = doc.createElement('link');
            clientStyles.rel = 'stylesheet';
            clientStyles.href = getRelativeUrl(config, clientStylesFilename);
            doc.head.append(clientStyles);
            const searchBox = doc.createElement('button');
            searchBox.classList.add('site-search-box');
            searchBox.type = 'button';
            searchBox.title = 'Press forward slash to search';
            searchBox.setAttribute('aria-label', 'Search');
            searchBox.setAttribute('data-plugin', 'search');
            searchBox.innerHTML = `
        <svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" fill="currentColor" class="bi bi-search" viewBox="0 0 16 16" aria-hidden="true">
          <path d="M11.742 10.344a6.5 6.5 0 1 0-1.397 1.398h-.001c.03.04.062.078.098.115l3.85 3.85a1 1 0 0 0 1.415-1.414l-3.85-3.85a1.007 1.007 0 0 0-.115-.1zM12 6.5a5.5 5.5 0 1 1-11 0 5.5 5.5 0 0 1 11 0z"></path>
        </svg>
        <span>Search</span>
      `;
            sidebarAddons?.append(searchBox);
            const title = stripWhitespace(opts.getTitle(doc));
            const description = stripWhitespace(opts.getDescription(doc));
            const headings = opts.getHeadings(doc).map(stripWhitespace);
            const content = stripWhitespace(opts.getContent(doc));
            const url = getRelativeUrl(config, page.outputFile);
            pagesToIndex.push({ page, title, description, headings, content, url });
            return doc;
        },
        afterAll: async (_pages, config) => {
            const searchIndexFilename = path.join(config.outputDir, config.assetFolderName, path.join(opts.searchDirName, 'search-index.json'));
            const searchDir = path.join(config.outputDir, config.assetFolderName, opts.searchDirName);
            const clientScriptFilename = path.join(searchDir, 'search-client.js');
            const clientStylesFilename = path.join(searchDir, 'search-client.css');
            const searchIndexUrl = getRelativeUrl(config, searchIndexFilename);
            const lunrSource = `${await fs.readFile('../node_modules/lunr/lunr.min.js', 'utf8')}\n`;
            const scriptSource = lunrSource +
                (await fs.readFile('../src/plugins/search/search-client.js', 'utf8')).replace(/SEARCH_INDEX_URL/g, searchIndexUrl);
            await fs.mkdir(path.dirname(clientScriptFilename), { recursive: true });
            await fs.copyFile('../src/plugins/search/search-client.css', clientStylesFilename);
            await fs.writeFile(clientScriptFilename, scriptSource, 'utf8');
            const searchIndex = lunr(async function () {
                let index = 0;
                this.ref('id');
                this.field('t', { boost: 10 });
                this.field('h', { boost: 5 });
                this.field('c');
                for (const page of pagesToIndex) {
                    if (opts.ignore(page.page))
                        continue;
                    this.add({ id: index, t: page.title, h: page.headings, c: page.content });
                    map[index] = { title: page.title, description: page.description, url: page.url };
                    index++;
                }
                await fs.mkdir(path.dirname(searchIndexFilename), { recursive: true });
                await fs.writeFile(searchIndexFilename, JSON.stringify({ searchIndex, map }), 'utf8');
                pagesToIndex = [];
            });
        }
    };
}
