import fs from 'fs/promises';
import frontMatter from 'front-matter';
import MarkdownIt from 'markdown-it';
import markdownItContainer from 'markdown-it-container';
import markdownItIns from 'markdown-it-ins';
import markdownItKbd from 'markdown-it-kbd';
import markdownItMark from 'markdown-it-mark';
const markdown = MarkdownIt({
    html: true,
    xhtmlOut: false,
    breaks: false,
    langPrefix: 'language-',
    linkify: false,
    typographer: false
});
markdown.use(markdownItIns);
markdown.use(markdownItKbd);
markdown.use(markdownItMark);
['tip', 'warning', 'danger'].forEach(type => {
    markdown.use(markdownItContainer, type, {
        render: function (tokens, idx) {
            if (tokens[idx].nesting === 1) {
                return `<div role="alert" class="callout callout--${type}">`;
            }
            return '</div>\n';
        }
    });
});
markdown.use(markdownItContainer, 'aside', {
    render: function (tokens, idx) {
        if (tokens[idx].nesting === 1) {
            return `<aside>`;
        }
        return '</aside>\n';
    }
});
markdown.use(markdownItContainer, 'cover', {
    render: function (tokens, idx) {
        if (tokens[idx].nesting === 1) {
            return `<div class="docs-cover">`;
        }
        return '</div>\n';
    }
});
markdown.use(markdownItContainer, 'details', {
    validate: (params) => params.trim().match(/^details\s+(.*)$/),
    render: (tokens, idx) => {
        const m = tokens[idx].info.trim().match(/^details\s+(.*)$/);
        if (tokens[idx].nesting === 1) {
            return `<details>\n<summary><span>${markdown.utils.escapeHtml(m[1])}</span></summary>\n`;
        }
        return '</details>\n';
    }
});
export async function parse(file) {
    const data = await fs.readFile(file, 'utf8');
    const { attributes, body } = frontMatter(data);
    return {
        frontMatter: attributes,
        content: body
    };
}
export function render(content, options) {
    const opts = {
        preserveHandlebars: true,
        ...options
    };
    if (!opts.preserveHandlebars) {
        return markdown.render(content);
    }
    let html = content
        .replace(/{{{/g, 'DOXICITY_RAW_START')
        .replace(/}}}/g, 'DOXICITY_RAW_END')
        .replace(/{{>/g, 'DOXICITY_PARTIAL_START')
        .replace(/{{#/g, 'DOXICITY_BLOCK_START')
        .replace(/{{#>/g, 'DOXICITY_BLOCK_PARTIAL_START')
        .replace(/{{/g, 'DOXICITY_START')
        .replace(/}}/g, 'DOXICITY_END');
    html = markdown.render(html);
    return html
        .replace(/DOXICITY_RAW_START/g, '{{{')
        .replace(/DOXICITY_RAW_END/g, '}}}')
        .replace(/DOXICITY_PARTIAL_START/g, '{{>')
        .replace(/DOXICITY_BLOCK_START/g, '{{#')
        .replace(/DOXICITY_BLOCK_PARTIAL_START/g, '{{#>')
        .replace(/DOXICITY_START/g, '{{')
        .replace(/DOXICITY_END/g, '}}');
}
